@@ -1,12 +1,12 @@
-#!/usr/bin/python2.7
+#!/usr/bin/python3
 # coding: utf-8
 #
 # Univention Management Console module:
 #  System Diagnosis UMC module
 #
-# Copyright 2017 Univention GmbH
+# Copyright 2017-2021 Univention GmbH
 #
-# http://www.univention.de/
+# https://www.univention.de/
 #
 # All rights reserved.
 #
@@ -29,21 +29,22 @@
 # You should have received a copy of the GNU Affero General Public
 # License with the Debian GNU/Linux or Univention distribution in file
 # /usr/share/common-licenses/AGPL-3; if not, see
-# <http://www.gnu.org/licenses/>.
+# <https://www.gnu.org/licenses/>.
 
 import subprocess
 import contextlib
 
 import univention.lib.admember
 import univention.config_registry
-from univention.management.console.modules.diagnostic import Critical
+from univention.management.console.modules.diagnostic import Critical, MODULE
 from univention.management.console.modules.diagnostic import util
 
 from univention.lib.i18n import Translation
 _ = Translation('univention-management-console-module-diagnostic').translate
 
 title = _('Check kerberos authenticated DNS updates')
-description = _('No errors occured.')
+description = _('No errors occurred.')
+run_descr = ['Checks if kerberos authenticated DNS updates']
 
 
 class UpdateError(Exception):
@@ -66,20 +67,21 @@ class KinitError(UpdateError):
 
 
 class NSUpdateError(UpdateError):
-	def __init__(self, hostname, domainname):
-		super(NSUpdateError, self).__init__(hostname, domainname)
-		self.hostname = hostname
+	def __init__(self, details, domainname):
+		super(NSUpdateError, self).__init__(details, domainname)
+		self.details = details
 		self.domainname = domainname
 
 	def __str__(self):
-		msg = _('`nsupdate` check for domain {domain} failed.')
-		return msg.format(domain=self.domainname)
+		msg = _('`nsupdate` check for domain {domain} failed ({details}).')
+		return msg.format(domain=self.domainname, details=self.details)
 
 
 @contextlib.contextmanager
 def kinit(principal, keytab=None, password_file=None):
 	auth = '--keytab={tab}' if keytab else '--password-file={file}'
 	cmd = ('kinit', auth.format(tab=keytab, file=password_file), principal)
+	MODULE.process('Running: %s' % (' '.join(cmd)))
 	try:
 		subprocess.check_call(cmd)
 	except subprocess.CalledProcessError:
@@ -89,45 +91,53 @@ def kinit(principal, keytab=None, password_file=None):
 		subprocess.call(('kdestroy',))
 
 
-def nsupdate(server, hostname, domainname):
-	process = subprocess.Popen(('nsupdate', '-g' , '-t', '15'), stdin=subprocess.PIPE,
-		stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
-	cmd = 'server {server}\nprereq yxdomain {host}.{domain}\nsend\n'
-	_ = process.communicate(cmd.format(server=server, host=hostname, domain=domainname))
+def nsupdate(server, domainname):
+	process = subprocess.Popen(('nsupdate', '-g', '-t', '15'), stdin=subprocess.PIPE, stdout=subprocess.PIPE, stderr=subprocess.STDOUT)
+	cmd_template = 'server {server}\nprereq yxdomain {domain}\nsend\nquit\n'
+	cmd = cmd_template.format(server=server, domain=domainname)
+	MODULE.process("Running: 'echo %s | nsupdate -g -t 15'" % (cmd,))
+
+	process.communicate(cmd)
 	if process.poll() != 0:
+		MODULE.error('NS Update Error at %s %s' % (server, domainname))
 		raise NSUpdateError(server, domainname)
 
 
-def get_server(config_registry):
-	server = config_registry.get('ldap/master')
+def get_dns_server(config_registry, active_services):
 	if config_registry.is_true('ad/member'):
 		ad_domain_info = univention.lib.admember.lookup_adds_dc()
-		return ad_domain_info.get('DC IP', server)
+		server = ad_domain_info.get('DC IP')
+	else:
+		hostname = config_registry.get('hostname')
+		domainname = config_registry.get('domainname')
+		if set(active_services) >= {'Samba 4', 'DNS'}:
+			server = ".".join([hostname, domainname])
+		else:
+			# TODO: Memberserver in Samba 4 domain
+			server = None
 	return server
 
 
 def check_dns_machine_principal(server, hostname, domainname):
 	with kinit('{}$'.format(hostname), password_file='/etc/machine.secret'):
-		nsupdate(server, hostname, domainname)
+		nsupdate(server, domainname)
 
 
 def check_dns_server_principal(hostname, domainname):
 	with kinit('dns-{}'.format(hostname), keytab='/var/lib/samba/private/dns.keytab'):
-		nsupdate(hostname, hostname, domainname)
+		nsupdate(hostname, domainname)
 
 
-def check_nsupdate(config_registry):
-	server = get_server(config_registry)
+def check_nsupdate(config_registry, server):
 	hostname = config_registry.get('hostname')
 	domainname = config_registry.get('domainname')
-	is_dc = config_registry.get('samba4/role') == 'DC'
 
 	try:
 		check_dns_machine_principal(server, hostname, domainname)
 	except UpdateError as error:
 		yield error
 
-	if is_dc:
+	if config_registry.get('samba4/role') == 'DC':
 		try:
 			check_dns_server_principal(hostname, domainname)
 		except UpdateError as error:
@@ -138,14 +148,22 @@ def run(_umc_instance):
 	config_registry = univention.config_registry.ConfigRegistry()
 	config_registry.load()
 
-	master_hostname = config_registry.get('ldap/master').split('.')[0]
-	if not util.is_service_active('Samba 4', master_hostname):
+	active_services = util.active_services()
+	if not set(active_services) & {'Samba 4', 'Samba 3'}:
 		return  # ddns updates are not possible
 
-	problems = list(check_nsupdate(config_registry))
+	try:
+		server = get_dns_server(config_registry, active_services)
+		if not server:
+			return
+	except NSUpdateError:
+		return  # ddns updates are not possible
+
+	problems = list(check_nsupdate(config_registry, server))
 	if problems:
-		ed = [_('Errors occured while running `kinit` or `nsupdate`.')]
+		ed = [_('Errors occurred while running `kinit` or `nsupdate`.')]
 		ed.extend(str(error) for error in problems)
+		MODULE.error('\n'.join(ed))
 		raise Critical(description='\n'.join(ed))
 
 
